@@ -10,13 +10,18 @@ require Exporter;
 use vars (qw/@ISA @EXPORT_OK/);
 @ISA = qw(Exporter);
 
-@EXPORT_OK = qw(stream_file);
+@EXPORT_OK = qw(stream stream_file);
 
-our $VERSION = '2.10';
+our $VERSION = '3.00_1';
 
-sub stream_file {
+sub stream {
     my ( $self, $file_or_fh, $bytes ) = @_;
+
     $bytes ||= 1024;
+
+    # Use unbuffered output, but return the state of $| to its previous state when we are done. 
+    local $| = 1;
+
     my ($fh, $basename);
     my  $size = (stat( $file_or_fh ))[7];
 
@@ -24,7 +29,7 @@ sub stream_file {
     if ( ref( \$file_or_fh ) eq 'SCALAR' ) {
         # They passed along a scalar, pointing to the path of the file
         # So we need to open the file
-        open($fh,"<$file_or_fh"  ) || return 0;
+        open($fh,"<$file_or_fh"  ) || die "failed to open file: $file_or_fh: $!";
         # Now let's go binmode (Thanks, William!)
         binmode $fh;
         $basename = basename( $file_or_fh );
@@ -46,7 +51,7 @@ sub stream_file {
 
     # Check for a existing type header set with or without a hypheout a hyphen
     unless ( $existing_headers{'-type'} ||  $existing_headers{'type'} ) {
-        my $mime_type;
+    	my $mime_type;
 
         eval {
             require File::MMagic;
@@ -75,26 +80,41 @@ sub stream_file {
         $self->header_add('-attachment' => $basename);
     }
 
-    unless ( $ENV{'CGI_APP_RETURN_ONLY'} ) {
-        $self->header_type( 'none' );
-        print $self->query->header( $self->header_props() );
-    }
+    $self->header_type( 'none' );
+    print $self->query->header( $self->header_props() );
 
     # This reads in the file in $byte size chunks
+    my $first;
     # File::MMagic may have read some of the file, so seek back to the beginning
-    my $output = "";
     seek($fh,0,0);
     while ( read( $fh, my $buffer, $bytes ) ) {
-        if ( $ENV{'CGI_APP_RETURN_ONLY'} ) {
-            $output .= $buffer;
-        } else {
-            print $buffer;
-        }
+		print $buffer;
     }
 
-    print '' unless $ENV{'CGI_APP_RETURN_ONLY'}; # print a null string at the end
+    print '';	# print a null string at the end
     close ( $fh );
-    return $ENV{'CGI_APP_RETURN_ONLY'} ? \$output : 1;
+    return 1;
+}
+
+# The old way. Requires manually calling error_mode() if there's a problem,
+# but error_mode() won't have access to "$@"
+sub stream_file {
+    my $self = shift;
+    my $out;
+
+    # Perhaps bad style to not use a method call here,
+    # But this keeps the legacy case working, where only stream_file() was exported. 
+    eval { stream($self,@_) }; 
+
+    # Starting with 3.0, we warn if there's a problem opening the file
+    # instead of ignoring the error. 
+    if ($@) {
+        warn $@;
+        return 0;
+    }
+    else {
+        return 1;
+    }
 }
 
 1;
@@ -105,7 +125,7 @@ CGI::Application::Plugin::Stream - CGI::Application Plugin for streaming files
 
 =head1 SYNOPSIS
 
-  use CGI::Application::Plugin::Stream (qw/stream_file/);
+  use CGI::Application::Plugin::Stream 'stream';
 
   sub runmode {
     # ...
@@ -115,11 +135,7 @@ CGI::Application::Plugin::Stream - CGI::Application Plugin for streaming files
 
     #...
 
-    if ( $self->stream_file( $file ) ) {
-      return;
-    } else {
-      return $self->error_mode();
-    }
+    return $self->stream($file);
   }
 
 =head1 DESCRIPTION
@@ -131,26 +147,22 @@ deliver to the user.
 
 The file is read and printed in small chunks to keep memory consumption down.
 
-This plugin is a consumer, as in your runmode shouldn't try to do any output or
-anything afterwards.  This plugin affects the HTTP response headers, so
-anything you do afterwards will probably not work.  If you pass along a
-filehandle, we'll make sure to close it for you.
-
-It's recommended that you increment $| (or set it to 1), which will
-autoflush the buffer as your application is streaming out the file.
+The C<stream()> method should be run as the last call in a a run mode, and
+affects the HTTP response headers.  If you pass along a filehandle, we'll make
+sure to close it for you.
 
 =head1 METHODS
 
-=head2 stream_file()
+=head2 stream()
 
-  $self->stream_file($fh);
-  $self->stream_file( '/path/to/file',2048);
+  $self->stream($fh);
+  $self->stream( '/path/to/file',2048);
 
 This method can take two parameters, the first the path to the file
 or a filehandle and the second, an optional number of bytes to determine
 the chunk size of the stream. It defaults to 1024.
 
-It will either stream a file to the user or return false if it fails, perhaps
+It will either stream a file to the user or die if it fails, perhaps
 because it couldn't find the file you referenced. 
 
 We highly recommend you provide a file name if passing along a filehandle, as we
@@ -159,7 +171,7 @@ won't be able to deduce the file name, and will use 'FILE' by default. Example:
  $self->header_add( -attachment => 'my_file.txt' );
 
 With both a file handle or file name, we will try to determine the correct
-content type by using File::MMagic. A default of 'application/octet-stream'
+content type by using L<File::MMagic>. A default of 'application/octet-stream'
 will be used if File::MMagic can't figure it out. 
 
 The size will be calculated and added to the headers as well. 
@@ -171,6 +183,13 @@ Again, you can set these explicitly if you want as well:
       -Content_Length	=>	42, # bytes
  );
 
+=head2 stream_file()  ( deprecated ) 
+
+This works the same as stream(), but returns false to indicate a problem.
+
+This is included because this was the original API. It does not give you
+access to "$@" if there is a problem. 
+
 =head1 AUTHOR
 
 Jason Purdy, E<lt>Jason@Purdy.INFOE<gt>,
